const inquirer = require('inquirer');
const request = require('./utils/request');
const log = require('./utils/log');
const syntax = require('./utils/syntax');
const context = require('./utils/context');
const colors = require('colors-console');

// 创建项目
function createProject (callback) {
  // 未登录提示
  if (context.getUserToken() == null) {
    log.warn('尊敬的用户，您当前没有登录，请勿填写项目敏感信息！如果您期望获取更强大的功能，可以使用coderd regis命令注册账号。当然，您也可以继续免登录体验完整的代码生成功能。')
  }
  // 定义问题
  const questions = [
    {
      name: 'enName',
      type: 'input',
      message: '请输入项目英文名称（必填）',
      validate(value) {
        if (value.trim().length === 0) {
          return '请输入项目英文名称'
        }
        if (!/^[0-9a-zA-Z_\\-]+$/g.test(value)) {
          return `项目英文名称仅允许使用英文、数字及特殊符号'-'和'_'`
        }
        return true
      }
    },
    {
      name: 'zhName',
      type: 'input',
      message: '请输入项目中文名称（必填）',
      validate(value) {
        if (value.trim().length === 0) {
          return '请输入项目中文名称'
        }
        return true
      }
    },
    {
      name: 'description',
      type: 'input',
      message: '请输入项目描述'
    }
  ];
  inquirer.prompt(questions)
    .then(answers => {
      answers.enName = answers.enName.trim();
      answers.zhName = answers.zhName.trim();
      answers.description = answers.description.trim();
      __confirmCreateProject(answers)
        .then(() => {
          callback && callback()
        })
        .catch(() => {})
    })
    .catch(e => {
      log.error(e)
    })
}

// 查询用户项目列表
function fetchProjectList () {
  log.info('正在查询...');
  request.post('/cli/project/list')
    .then(projects => {
      for (const project of projects) {
        log.info(`${project.enName + (project.projectDesc === '' ? '' : (': ' + project.projectDesc))}${project.id === context.getProject() ? colors('green',' (default)') : ''}`);
      }
      if (projects.length === 0) {
        log.tip('你可以使用coderd project -n来创建一个新项目')
      }
      log.success(`查询完成，共查询到 ${projects.length} 个项目`)
    })
    .catch(e => {
      log.error(e)
    })
}

// 设置项目
function setProject (enName) {
  log.info('正在设置默认项目...');
  request.post('/cli/project/query', {enName})
    .then(project => {
      context.setProject(project.id);
      log.success(`已将 ${project.enName} 设置为默认项目`)
    })
    .catch(e => {
      // 未找到项目
      if (e.code === '010010') {
        log.warn(`找不到 ${enName} 项目，你可以使用coderd project -l来查询你拥有的所有项目`);
        return
      }
      log.error(e);
    })
}

// 确认项目创建
function __confirmCreateProject (project) {
  log.info('正在创建项目...');
  return request.post('/cli/project/create', project)
    .then(data => {
      context.setProject(data.id);
      log.success(`项目创建成功并已设置为默认项目`);
      return Promise.resolve();
    })
    .catch(e => {
      if (e.code === '999998') {
        log.warn('创建失败，项目名称已存在');
        return Promise.reject();
      }
      if (e.code === '000102') {
        log.warn('创建失败，当前账户项目数量已达上限！开通会员可立享项目无上限。');
        return Promise.reject();
      }
      log.error(e);
      return Promise.reject();
    })
}

module.exports = {
  createProject,
  handleCommand (command, option, args) {
    if (command !== 'use' && command !== 'project') {
      return false
    }
    // 设置默认项目
    if (command === 'use') {
      syntax.check(args, option, [1,2]);
      let enName = args[1] || option.project;
      if (enName == null) {
        syntax.printSyntaxError();
        process.exit()
      }
      setProject(enName);
      return true;
    }
    if (command === 'project') {
      syntax.check(args, option, [1]);
      // 新建项目
      if (option.new) {
        createProject();
        return true;
      }
      fetchProjectList();
      return true;
    }
  }
};
