const inquirer = require('inquirer');
const Rx = require('rxjs');
const request = require('./utils/request');
const runner = require('./utils/runner');
const context = require('./utils/context');
const str = require('./utils/string');
const log = require('./utils/log');
const project = require('./project');

// 获取年中的周索引
const getWeekOfYear = date => {
  const startDate = new Date(date.getFullYear(), 0, 1);
  const d = Math.round((date.valueOf() - startDate.valueOf()) / 86400000);
  return Math.ceil((d + (startDate.getDay() + 1 - 1)) / 7)
};

// 获取月中的周索引
const getWeekOfMonth = date => {
  const w = date.getDay();
  const d = date.getDate();
  return Math.ceil((d + 6 - w) / 7);
};

// 日期属性
const now = new Date();
const year = now.getFullYear();
const month = now.getMonth() + 1;
const day = now.getDate();
const hours = now.getHours();
const hours12 = hours % 12
const minutes = now.getMinutes();
const seconds = now.getSeconds();
const milliseconds = now.getMilliseconds();
const dateProps = {
  _yyyy: now.getFullYear(),
  _yy: ('' + year).substr(2, 2),
  _MM: ('0' + month).substr(('0' + month).length - 2),
  _dd: ('0' + day).substr(('0' + day).length - 2),
  _weekOfMonth: getWeekOfMonth(now),
  _weekOfYear: getWeekOfYear(now),
  _HH: ('0' + hours).substr(('0' + hours).length - 2),
  _hh: ('0' + hours12).substr(('0' + hours12).length - 2),
  _mm: ('0' + minutes).substr(('0' + minutes).length - 2),
  _ss: ('0' + seconds).substr(('0' + seconds).length - 2),
  _sss: ('00' + milliseconds).substr(('00' + milliseconds).length - 3)
};
// 编译数据
const compileData = {
  // 系统属性数据
  systemProps: {
    _author: 'CodeRd',
    _email: 'http://coderd.adjustrd.com',
    _mobile: '',
    _tables: [],
    ...dateProps
  },
  // 模版组属性数据
  templateGroupProps: {},
  // 模版属性数据
  templateProps: {}
};

// 保存问题答案
const answers = [];

// 获取问题类型
function getType(prop) {
  // 文本
  if (prop.inputType === 'TEXT' || prop.inputType === 'TEXTAREA') {
    return 'input';
  }
  // 多选
  if (prop.inputType === 'SELECT_MULTIPLE') {
    return 'checkbox';
  }
  // 单选
  if (prop.inputType === 'SELECT_SINGLE') {
    return 'list';
  }
  // 布尔
  if (prop.inputType === 'BOOLEAN') {
    return 'list';
  }
  // 数字
  if (prop.inputType === 'NUMBER') {
    return 'input';
  }
  return 'unknown';
}

// 获取属性选项
const getChoices = function (prop) {
  if (prop.inputType === 'BOOLEAN') {
    return [
      {
        name: '是',
        value: true
      },
      {
        name: '否',
        value: false
      }
    ]
  }
  return prop.items.map(item => {
    return {
      name: item.name + (item.onlyVip ? '(VIP)' : ''),
      value: item.value
    }
  })
};

// freemarker解析
const __freemarker = async function(templateStr) {
  if (templateStr == null || templateStr === '') {
    return { success: true, data: '' }
  }
  // 如果为非字符串
  if (typeof templateStr !== 'string') {
    return { success: true, data: templateStr }
  }
  // 只有在包含表达式的时才解析
  if (templateStr.indexOf('$') === -1) {
    return { success: true, data: templateStr }
  }
  const tempCompileData = {
    ...compileData.systemProps,
    ...compileData.templateGroupProps
  };
  return await request.post('/freemarker/parse', {
    templateStr, datas: tempCompileData
  })
    .then(resp => {
      return { success: true, data: resp }
    })
    .catch(e => {
      return { success: false, data: templateStr, error: e }
    })
};

// 编译属性
const compileProps = async function(props) {
  // 清理系统数据
  compileData.systemProps._basePath = null;
  compileData.systemProps._projectName = null;
  // 添加系统数据
  compileData.systemProps._basePath = 'codes';
  compileData.systemProps._projectName = compileData.systemProps._basePath;

  // 编译模版组属性
  compileData.templateGroupProps = {};
  for (const prop of props) {
    // 单选/多选/布尔/整数，无需编译，取值本身
    if (prop.inputType === 'SELECT_SINGLE' || prop.inputType === 'SELECT_MULTIPLE' || prop.inputType === 'BOOLEAN' || prop.inputType === 'NUMBER') {
      compileData.templateGroupProps[prop.variableName] = prop.value;
      continue;
    }
    // 文本/文本域，需要编译
    compileData.templateGroupProps[prop.variableName] = (await __freemarker(prop.value)).data;
  }
};

// 获取问题
function getQuestions (props) {
  return props.filter(p => p.isShow).map(prop => {
    const question = {
      id: prop.id,
      type: getType(prop),
      name: prop.variableName,
      message: prop.name,
      default: prop.value
    };
    // 单选/多选/布尔处理
    if (prop.inputType === 'SELECT_SINGLE' || prop.inputType === 'SELECT_MULTIPLE' || prop.inputType === 'BOOLEAN') {
      question.choices = getChoices(prop)
    }
    // 校验
    // - 数字校验
    if (prop.inputType === 'NUMBER') {
      question.validate = function(value) {
        if (prop.required) {
          if (value.trim() === '') {
            return `请输入${prop.name}`
          }
        }
        if (isNaN(value)) {
          return '请输入一个整数值'
        }
        return true
      }
    }
    // - 必填校验
    else {
      question.validate = function(value) {
        if (prop.required) {
          // 多选
          if (prop.inputType === 'SELECT_MULTIPLE') {
            if (value.length === 0) {
              return '请至少选择一个选项'
            }
          }
          // 其他类型
          else if (value.trim() === '') {
            return `请输入${prop.name}`
          }
        }
        return true
      }
    }
    return question
  })
}

// 根据属性初始化答案
function initAnswerByProps (props) {
  answers.push.apply(answers, props)
}

// 根据项目配置初始化value
async function initPropsByProject (props) {
  await request.post('/cli/project/query', {
    id: context.getProject()
  })
    .then(data => {
      return data.configJson
    })
    .then(data => {
      const valueConfig = data == null || data === '' ? {} : JSON.parse(data);
      for (const prop of props) {
        // 文本/文本域/数字/布尔
        if (
          prop.inputType === 'TEXT' || prop.inputType === 'TEXTAREA' || prop.inputType === 'NUMBER' || prop.inputType === 'BOOLEAN'
        ) {
          prop.value = prop.defaultValue;
          if (prop.isShow || prop.isShowLogic) {
            prop.value = valueConfig[prop.variableName] == null
              ? prop.defaultValue
              : valueConfig[prop.variableName]
          }
          continue
        }
        // 单选：过滤选项值
        if (prop.inputType === 'SELECT_SINGLE') {
          prop.value = prop.defaultValue != null && prop.defaultValue !== '' ? prop.defaultValue : '';
          // 如果配置中存在值 && 值在选项中，则选中配置值
          if ((prop.isShow || prop.isShowLogic) && valueConfig[prop.variableName] != null && prop.items.findIndex(item => item.value === valueConfig[prop.variableName]) > -1) {
            prop.value = valueConfig[prop.variableName]
          }
          continue
        }
        // 多选：数组结构
        if (prop.inputType === 'SELECT_MULTIPLE') {
          prop.value = prop.defaultValue != null && prop.defaultValue !== '' ? prop.defaultValue.split(',') : [];
          if ((prop.isShow || prop.isShowLogic) && valueConfig[prop.variableName] != null) {
            // 过滤不存在的选项
            prop.value = valueConfig[prop.variableName].filter(value => prop.items.findIndex(item => item.value === value) > -1)
          }
        }
      }
      return Promise.resolve(props)
    })
    .catch(e => {
      // 找不到项目
      if (e.code === '010010') {
        log.warn('初始化项目配置失败，找不到项目信息，请确认项目是否存在');
        process.exit();
      }
      // 不属于项目成员
      if (e.code === '000104') {
        log.warn('生成失败！您不属于该项目成员。')
        process.exit();
      }
      log.error(e);
      process.exit();
    })
}

// 准备
function prepare (alias, command, versionNo, option, logicFunc) {
  log.info('正在获取模版信息...');
  request.post(option.test ? '/cli/template/draft' : '/cli/template/version', {
    alias, command, versionNo
  })
    .then(data => {
      log.info('模版信息获取完成!');
      // 未携带数据模型
      if (option.resources == null || option.resources === '') {
        // 如果生成时需展示数据模型，则查询所有数据模型
        const settings = JSON.parse(data.settings)
        if (settings.generate.showDataPanel) {
          request.post('/cli/project/query/dms', {projectId: context.getProject()})
            .then(dms => {
              if (dms.length === 0) {
                log.warn('生成失败！该项目下没有可用的数据模型。请前往http://coderd.adjustrd.com/配置！')
                process.exit();
              }
              const choices = []
              for (const dm of dms) {
                choices.push({
                  name: `${dm.name}${dm.remark === '' ? '' : ('/' + dm.remark)}`,
                  value: dm.name
                })
              }
              inquirer.prompt({
                name: 'resources',
                type: 'checkbox',
                choices: choices,
                message: '请选择数据模型',
                pageSize: 10 // 一页展示多少条
              },)
                .then(answers => {
                  if (answers.resources.length === 0) {
                    log.info('正在生成代码...');
                    log.success(`共生成 0 个文件`)
                    process.exit();
                  }
                  option.resources = answers.resources.join(',')
                  logicFunc(data)
                })
                .catch(e => {
                  log.error(e)
                })
            })
            .catch(e => {
              // 不属于项目成员
              if (e.code === '000104') {
                log.warn('生成失败！您不属于该项目成员。')
                process.exit();
              }
              log.error(e);
            })
          return
        }
      }
      logicFunc(data)
    })
    .catch(e => {
      if (e.code === '000004') {
        log.warn(`找不到模版 {alias: ${alias}, function: ${command == null ? '' : command}, versionNo: ${versionNo == null ? '' : versionNo}}`)
        return;
      }
      if (e.code === '010003') {
        log.warn(`找不到模版 {alias: ${alias}} 的功能命令 '${command}'`);
        return
      }
      if (e.code === '010004') {
        log.warn(`未指定模版 {alias: ${alias}} 的功能命令`);
        return
      }
      log.error(e)
    })
}

// 初始化Runner
function init (alias, command, versionNo, option) {
  log.debug(`init runner`)
  log.debug(`\talias: ${alias}`)
  log.debug(`\tcommand: ${command}`)
  log.debug(`\tversionNo: ${versionNo}`)
  log.debug(`\toption: ${JSON.stringify(option)}`)
  prepare(alias, command, versionNo, option, (data) => {
    const template = {
      versionId: option.test ? '' : data.id,
      draftId: option.test ? data.id : '',
    };
    context.setTemplate({alias, command, versionNo, option});
    request.get(`/runner/data?${option.test ? 'draftId' : 'versionId'}=${data.id}`)
      .then(async data => {
        const props = data.props;
        // 根据项目配置初始化属性值
        await initPropsByProject(props);
        // 初始化问题答案
        initAnswerByProps(props);
        // 如果为使用最后一次生成配置进行生成，则直接生成
        if (option.remember) {
          runner.generate(answers, option.resources, template, option.force);
          return
        }
        // 获取问题
        let questions = getQuestions(props);
        if (questions.length === 0) {
          runner.generate(answers, option.resources, template, option.force);
          return
        }
        let questionIndex = 1;
        const prompts = new Rx.Subject();
        // 监听答案输入
        inquirer.prompt(prompts).ui.process.subscribe(async (onEachAnswer) => {
          const prop = props.find(prop => prop.variableName === onEachAnswer.name);
          if (onEachAnswer.answer.constructor === Array) {
            prop.value = onEachAnswer.answer
          } else if (prop.inputType === 'NUMBER') {
            prop.value = str.isBlank(onEachAnswer.answer) ? prop.value : parseInt(onEachAnswer.answer)
          } else {
            prop.value = str.isBlank(onEachAnswer.answer) ? prop.value : onEachAnswer.answer
          }
          // 当选择类型的值确认后触发显示逻辑编译
          if (prop.inputType === 'SELECT_SINGLE' || prop.inputType === 'SELECT_MULTIPLE' || prop.inputType === 'BOOLEAN') {
            const hasIsShowLogicCode = props.findIndex(p => p.isShowLogic !== null && p.isShowLogic !== '') !== -1;
            if (hasIsShowLogicCode) {
              await compileProps(props)
                .then(async () => {
                  for (const prop of props) {
                    if (prop.isShowLogic == null || prop.isShowLogic === '') {
                      continue;
                    }
                    const result = await __freemarker(prop.isShowLogic);
                    if (result.success) {
                      prop.isShow = result.data.trim().toLowerCase() === 'true';
                      // 隐藏时变为默认值
                      if (!prop.isShow) {
                        prop.value = prop.defaultValue
                      }
                      continue
                    }
                    prop.isShow = false;
                    let errorMessage = `属性 '${prop.name}' 的【显示逻辑】代码错误`;
                    for (const tplName in result.error.edata) {
                      errorMessage += `\n代码: \n"${tplName}"`;
                      errorMessage += '\n\n错误信息：\n';
                      errorMessage += result.error.edata[tplName]
                    }
                    prompts.complete();
                    break
                  }
                });
              // 重新整理问题
              questions = getQuestions(props)
            }
          }
          // 判断问题结束（需要等问题重新整理后再判断）
          if (questionIndex === questions.length) {
            prompts.complete()
            runner.generate(answers, option.resources, template, option.force);
            return
          }
          prompts.next(questions[questionIndex]);
          questionIndex++
        });
        // 监听后再初始化第一个问题，否则不生效
        prompts.next(questions[0])
      })
      .catch(e => {
        log.error(e)
      })
  })
}

module.exports = {
  handleCommand (command, option, args) {
    if (args.length !== 1 && args.length !== 2) {
      return false;
    }
    const projectId = context.getProject();
    // coderd run，根据当前项目下最后一次的生成配置进行生成
    if (args.length === 1 && command === 'run') {
      const rememberTemplate = context.getTemplate();
      if (rememberTemplate == null || rememberTemplate.alias == null) {
        log.error('当前目录下暂未记忆生成配置，你可以使用coderd run <alias>指定模版别名进行记忆生成');
        process.exit();
      }
      const templateAlias = rememberTemplate.alias;
      const opt = {};
      for (const key in option) {
        opt[key] = option[key] == null ? rememberTemplate.option[key] : option[key];
      }
      opt.remember = true
      if (str.isBlank(projectId)) {
        project.createProject(() => {
          init(templateAlias, rememberTemplate.command, rememberTemplate.versionNo, opt);
        })
      } else {
        init(templateAlias, rememberTemplate.command, rememberTemplate.versionNo, opt);
      }
      return true
    }
    // coderd run <alias>，根据使用过的模版配置进行生成
    if (args.length === 2 && command === 'run') {
      const rememberTemplate = context.getTemplate(args[1]);
      if (rememberTemplate == null) {
        log.error(`未找到模版{ alias: ${args[1]} }的生成配置`);
        process.exit();
      }
      const templateAlias = rememberTemplate.alias;
      const opt = {};
      for (const key in option) {
        opt[key] = option[key] == null ? rememberTemplate.option[key] : option[key];
      }
      opt.remember = true
      if (str.isBlank(projectId)) {
        project.createProject(() => {
          init(templateAlias, rememberTemplate.command, rememberTemplate.versionNo, opt);
        })
      } else {
        init(templateAlias, rememberTemplate.command, rememberTemplate.versionNo, opt);
      }
      return true
    }
    // 无功能命令，如coderd vue
    let templateAlias = command;
    let versionNo = null;
    if (command.indexOf('@')) {
      templateAlias = command.split('@')[0];
      versionNo = command.split('@')[1];
    }
    if (args.length === 1) {
      if (str.isBlank(projectId)) {
        project.createProject(() => {
          init(templateAlias, null, versionNo, option);
        })
      } else {
        init(templateAlias, null, versionNo, option);
      }
      return true
    }
    // 有功能命令，如coderd springboot build
    if (args.length === 2) {
      let functionCommand = args[1]
      if (functionCommand === 'i') {
        functionCommand = 'interface'
      } else if (functionCommand === 'p') {
        functionCommand = 'page'
      } else if (functionCommand === 'b') {
        functionCommand = 'build'
      } else if (functionCommand === 'bs') {
        functionCommand = 'build-server'
      } else if (functionCommand === 'bc') {
        functionCommand = 'build-client'
      }
      if (str.isBlank(projectId)) {
        project.createProject(() => {
          init(templateAlias, functionCommand, versionNo, option);
        })
      } else {
        init(templateAlias, functionCommand, versionNo, option);
      }
      return true
    }
  }
};
