const log = require('./utils/log');
const syntax = require('./utils/syntax');
const context = require('./utils/context');
const request = require('./utils/request');
const DBUtil = require('./utils/db');
const inquirer = require('inquirer');

// 输出所有表
function printTables () {
  log.info('正在获取表记录...');
  DBUtil.getTables(({tables}) => {
    for (const table of tables) {
      log.info(`${table.name}${table.remark === '' ? '' : ('/' + table.remark)}`);
    }
    log.success(`共获取到 ${tables.length} 张表记录`);
  })
}

// 同步表
const syncTableData = {
  total: null,
  failedCount: 0,
  successCount: 0
};

// 选择表
function selectTables () {
  log.info('正在获取表记录，请稍后...')
  DBUtil.getTables(({tables}) => {
    log.success(`共获取到 ${tables.length} 张表记录`);
    const choices = []
    for (const table of tables) {
      choices.push({
        name: table.name + (table.remark === '' ? '' : ` (${table.remark})`),
        value: table.name
      })
    }
    inquirer.prompt({
      name: 'tables',
      type: 'checkbox',
      choices: choices,
      message: '请选择需要同步的表',
      pageSize: 10 // 一页展示多少条
    },)
      .then(answers => {
        if (answers.tables == null || answers.tables.length === 0) {
          log.success(`同步总数: 0，成功: 0`)
          return
        }
        const tables = answers.tables.map(tableName => {
          return {
            name: tableName
          }
        }).sort()
        syncTableData.total = tables.length;
        syncTable(tables[0].name, false, tables.slice(1))
      })
      .catch(e => {
        log.error(e)
      })
  })

}

/**
 * 同步指定表
 * @param tableName 表名
 * @param force 是否强制同步
 * @param unsyncTables 未同步的表集合（存在该值时，将在当前表同步完成后自动同步下一张表）
 */
function syncTable (tableName, force = false, unsyncTables = []) {
  // 指定表名同步
  log.info(`正在同步 ${tableName} 表...`);
  DBUtil.getTables(({tables}) => {
    const table = tables.find(t => t.name === tableName);
    if (table == null) {
      log.warn(`未找到 ${tableName} 表`);
      return
    }
    DBUtil.getFields(tableName, ({fields}) => {
      request.post('/cli/dm/sync', {
        force,
        projectId: context.getProject(),
        name: table.name,
        originName: table.name,
        remark: table.remark,
        props: fields
      })
        .then(() => {
          log.info(`同步完成`);
          syncTableData.successCount++
        })
        .catch(e => {
          syncTableData.failedCount++;
          // 未登录
          if (e.code === '000002') {
            log.error(`表同步失败！当前登录信息已过期，您可以使用coderd login进行登录后再试。`);
            unsyncTables = []
            process.exit()
            return
          }
          // 未找到项目信息
          if (e.code === '000100') {
            log.error(`表同步失败！项目不存在或未指定默认项目，您可以使用coderd project查看您账号下的项目，并通过coderd use {项目英文名称}指定默认项目，或使用coderd project -n来创建一个新项目。`);
            unsyncTables = []
            process.exit()
            return
          }
          // 不属于项目成员
          if (e.code === '000104') {
            log.error('表同步失败！您不属于该项目成员。')
            unsyncTables = []
            process.exit()
            return
          }
          log.error(e)
          // log.error(`${tableName} 表同步失败`);
        })
        .finally(() => {
          if (unsyncTables.length > 0) {
            setTimeout(() => {
              syncTable(unsyncTables[0].name, force, unsyncTables.slice(1))
            }, 500)
          }
          else {
            syncTableData.total = syncTableData.total == null ? 1 : syncTableData.total;
            if (syncTableData.total === syncTableData.successCount) {
              log.success(`同步总数: ${syncTableData.total}，成功: ${syncTableData.successCount}`)
            }else {
              log.warn(`同步总数: ${syncTableData.total}，成功: ${syncTableData.successCount}，失败: ${syncTableData.failedCount}`)
            }
          }
        })
    })
  })
}

// 同步所有表
function syncAllTables () {
  log.info('正在获取表记录，请稍后...')
  DBUtil.getTables(({tables}) => {
    log.success(`共获取到 ${tables.length} 张表记录`);
    syncTableData.total = tables.length;
    syncTable(tables[0].name, false, tables.slice(1))
  })
}

module.exports = {
  handleCommand (command, option, args) {
    if (command !== 'table') {
      return false
    }
    syntax.check(args, option, [1]);
    // 验证数据库
    if (!context.checkDatabase()) {
      process.exit();
    }
    // 同步所有表
    if (option.sync && option.all) {
      const projectId = context.getProject();
      if (projectId == null || projectId === '') {
        log.error(`当前没有指定默认项目，你可以通过coderd project -l查询你拥有的所有项目，并通过coderd use -p <project-english-name>来指定默认项目`);
        process.exit();
      }
      syncAllTables()
      return true
    }
    // 选择 & 同步表结构
    if (option.sync === true) {
      const projectId = context.getProject();
      if (projectId == null || projectId === '') {
        log.error(`当前没有指定默认项目，你可以通过coderd project -l查询你拥有的所有项目，并通过coderd use -p <project-english-name>来指定默认项目`);
        process.exit();
      }
      selectTables();
      return true
    }
    // 同步指定表
    if (option.sync) {
      const projectId = context.getProject();
      if (projectId == null || projectId === '') {
        log.error(`当前没有指定默认项目，你可以通过coderd project -l查询你拥有的所有项目，并通过coderd use -p <project-english-name>来指定默认项目`);
        process.exit();
      }
      syncTable(option.sync, false)
      return true
    }
    // 打印所有表
    printTables();
    return true
  }
};
