const HOME = `${process.env.HOME || process.env.SystemDrive + process.env.HOMEPATH }/.coderd`;
const HOME_FILE = `${HOME}/coderd.json`;
const PROJECT_FILE = `${process.cwd()}/coderd.json`;
const fs = require('fs');
const log = require('./log');
const contextConfig = require('../config/context');
const str = require('./string');

// 初始化Home文件
function initHomeContext (userToken=null) {
  if (!fs.existsSync(HOME)) {
    fs.mkdirSync(HOME)
  }
  __writeHomeContext({token: userToken})
}

// 初始化项目文件
function initProjectContext () {
  __writeProjectContext({});
}

// 获取全局上下文
function __getHomeContext () {
  if (!fs.existsSync(HOME_FILE)) {
    return JSON.parse(JSON.stringify(contextConfig.homeContext))
  }
  return require(HOME_FILE)
}

// 获取项目上下文
function __getProjectContext () {
  if (!fs.existsSync(PROJECT_FILE)) {
    return JSON.parse(JSON.stringify(contextConfig.projectContext))
  }
  return require(PROJECT_FILE)
}

// 写入全局上下文
function __writeHomeContext (overrideContext) {
  if (!fs.existsSync(HOME)) {
    fs.mkdirSync(HOME)
  }
  const homeContext = __getHomeContext();
  Object.assign(homeContext, overrideContext);
  try {
    fs.writeFileSync(HOME_FILE, JSON.stringify(homeContext, null, 2));
  } catch (e) {
    log.error('无法写入上下文内容，请尝试使用超级管理员权限打开终端后重试。')
    process.exit();
  }
}

// 写入项目上下文
function __writeProjectContext (overrideContext) {
  // 全局上下文
  if (!fs.existsSync(HOME_FILE)) {
    initHomeContext()
  }
  // 项目上下文
  const projectContext = __getProjectContext();
  Object.assign(projectContext, overrideContext);
  try {
    fs.writeFileSync(PROJECT_FILE, JSON.stringify(projectContext, null, 2));
  } catch (e) {
    log.error('无法写入上下文内容，请尝试使用超级管理员权限打开终端后重试。')
    process.exit();
  }
}

module.exports = {
  initHomeContext,
  initProjectContext,
  getUserToken () {
    return __getHomeContext().token
  },
  // 获取是否为debug模式
  getIsDebug () {
    return __getProjectContext().debug
  },
  // 设置是否为debug模式
  setIsDebug (debug) {
    __writeProjectContext({debug})
  },
  getProject () {
    return __getProjectContext().project
  },
  setProject (project) {
    __writeProjectContext({project})
  },
  checkDatabase () {
    const database = this.getDatabase()
    if (database == null || str.isBlank(database.host) || str.isBlank(database.port) || str.isBlank(database.name) || str.isBlank(database.user)) {
      log.error('数据库信息配置不完整，你可以使用coderd db命令配置完整信息！');
      return false
    }
    return true
  },
  getDatabase () {
    const database = __getProjectContext().database
    if (database != null) {
      return database
    }
    return {
      host: "",
      port: "",
      name: "",
      user: "",
      password: ""
    }
  },
  setDatabase (database) {
    __writeProjectContext({database})
  },
  /**
   * @param alias 模版别名，当为null时表示查找第一条
   */
  getTemplate (alias=null) {
    // 获取项目模版配置
    if (alias == null) {
      return __getProjectContext().template
    }
    // 从全局上下文中获取模版配置
    const templates = __getHomeContext().templates
    if (templates == null || templates.length === 0) {
      return null;
    }
    for (const template of templates) {
      if (template.alias === alias) {
        return template;
      }
    }
    return null;
  },
  setTemplate (template) {
    template = JSON.parse(JSON.stringify(template))
    // 不记录remember选项
    if (template.option) {
      delete template.option.remember
    }
    // 写入项目上下文
    __writeProjectContext({template})
    // 写入全局上下文
    let templates = __getHomeContext().templates
    if (templates == null) {
      templates = []
    }
    let override = false
    for (const tpl of templates) {
      if (tpl.alias === template.alias) {
        Object.assign(tpl, template);
        override = true;
        break;
      }
    }
    if (!override) {
      templates.push(template)
    }
    __writeHomeContext({templates})
  },
  getIgnores () {
    const ignores = __getHomeContext().ignores
    const projectIgnores = __getProjectContext().ignores
    if (projectIgnores != null && projectIgnores.indexOf(Array)) {
      ignores.push.apply(ignores, projectIgnores)
    }
    return ignores
  },
  /**
   * 设置提醒版本，提醒过的版本不再提醒
   */
  setRemindVersion (versionNo) {
    __writeHomeContext({
      'remind-version': versionNo
    })
  },
  getRemindVersion () {
    return this.get('remind-version')
  },
  /**
   * @param key 配置项
   */
  get (key) {
    return __getHomeContext()[key]
  }
};
