const request = require('./request');
const log = require('./log');
const context = require('./context');
const fs = require('fs');
const inquirer = require('inquirer');
const Rx = require('rxjs');
const colors = require('colors-console');
// 生成总数
let totalFileCount = 0
// 统计写入的文件
let writeFileCount = 0
// 已存在的文件
const existsFiles = []

// 读取目录
function getHolderFile (dirpath) {
  // 不扫描忽略的目录
  const ignoreDirs = context.getIgnores('ignores');
  for(const ignore of ignoreDirs) {
    if (new RegExp(ignore).test(dirpath)) {
      return
    }
  }
  const files = fs.readdirSync(dirpath);
  for(const fileName of files) {
    const filePath = `${dirpath}/${fileName}`;
    const stat = fs.statSync(filePath);
    if (stat.isFile()) {
      // 不扫描忽略的文件
      let jump = false;
      for(const ignore of ignoreDirs) {
        if (new RegExp(ignore).test(filePath)) {
          jump = true;
          break
        }
      }
      if (jump) {
        continue
      }
      // 找到包含占位符的文件并返回
      let content = fs.readFileSync(filePath)
      if (content.toString().indexOf(context.get("holder")) > -1) {
        return {
          filePath,
          content
        }
      }
      continue
    }
    // 如果是目录，则继续查找
    if (stat.isDirectory()) {
      const file = getHolderFile(filePath)
      if (file != null) {
        return file
      }
    }
  }
}

// 写入文件
function writeFile (file, path) {
  log.info(`${file.filePath + '/' + file.fileName} ${colors('yellow', file.fileSizeDisplay)}`);
  writeFileCount++
  if (file.fileViewType === 'image') {
    const imageBuffer = Buffer.from(file.content, 'base64');
    fs.writeFileSync(path, imageBuffer)
    return
  }
  // - 文件
  if (file.fileViewType === 'file') {
    const fileBuffer = Buffer.from(file.content, 'hex');
    fs.writeFileSync(path, fileBuffer)
    return
  }
  // - 代码文件
  fs.writeFileSync(path, file.content)
}

// 处理已存在的文件
function handleExistsFile () {
  log.debug(`start handle exists files.`)
  if (existsFiles.length === 0) {
    log.debug(`end handle exists files.`)
    return
  }
  // 构造问题
  const questions = []
  for (let i = 0; i < existsFiles.length; i++) {
    const file = existsFiles[i]
    questions.push({
      _file: file,
      name: 'handler' + i, // 答案的key不能一样，否则无法进行二次提问
      type: 'list',
      message: `${file.path}文件已存在，您希望如何处理呢？`,
      choices: [
        {
          name: '覆盖',
          value: 'rewrite'
        },
        {
          name: '跳过',
          value: 'jump'
        },
        {
          name: '全部覆盖',
          value: 'rewrite_all'
        },
        {
          name: '全部跳过',
          value: 'jump_all'
        }
      ]
    })
  }
  const prompts = new Rx.Subject();
  let currentQuestionIndex = 0
  let total = 0
  let rewriteCount = 0
  let jumpCount = 0
  // 监听答案输入
  inquirer.prompt(prompts).ui.process.subscribe(async (onEachAnswer, question) => {
    const currentQuestion = questions[currentQuestionIndex]
    // 覆盖
    if (onEachAnswer.answer === 'rewrite') {
      log.debug(`select rewrite.`)
      writeFile(currentQuestion._file.file, currentQuestion._file.path)
      total++
      rewriteCount++
    }
    // 跳过
    else if (onEachAnswer.answer === 'jump') {
      log.debug(`select jump.`)
      total++
      jumpCount++
    }
    // 全部覆盖
    else if (onEachAnswer.answer === 'rewrite_all') {
      log.debug(`select rewrite_all.`)
      for (let i = total; i < existsFiles.length; i++) {
        const file = existsFiles[i]
        writeFile(file.file, file.path)
      }
      rewriteCount = rewriteCount + questions.length - total
      total = questions.length
      prompts.complete()
    }
    // 全部跳过
    else if (onEachAnswer.answer === 'jump_all') {
      log.debug(`select jump_all.`)
      jumpCount = jumpCount + questions.length - total
      total = questions.length
      prompts.complete()
    }
    // 询问下一个文件的处理
    if (questions.length > total) {
      log.debug(`ask for next file.`)
      currentQuestionIndex++
      questions[currentQuestionIndex].name = questions[currentQuestionIndex].name + currentQuestionIndex
      prompts.next(questions[currentQuestionIndex])
    } else {
      log.success(`共生成 ${totalFileCount} 个文件，覆盖写入 ${rewriteCount} 个文件, 跳过 ${jumpCount} 个文件`)
      prompts.complete()
      log.debug(`end handle exists files.`)
    }
  })
  prompts.next(questions[currentQuestionIndex])
}

module.exports = {

  // 处理文件
  handleFile (file, force = false) {
    const fullPath = file.filePath + '/' + file.fileName;
    const filePaths = fullPath.split('/');
    log.debug(`handle file ${fullPath}`)
    let path = null;
    for (let i = 0; i < filePaths.length; i++) {
      // 项目目录不处理
      if (i === 0) {
        continue
      }
      path = path == null ? filePaths[i] : path + '/' + filePaths[i]
      // 创建目录
      if (i < filePaths.length - 1) {
        if (fs.existsSync(path)) {
          continue
        }
        log.debug(`create directory ${path}`)
        fs.mkdirSync(path);
        continue
      }
      // 文件已存在
      if (fs.existsSync(path) && !force) {
        log.debug(`${path} already exists.`)
        existsFiles.push({
          file,
          path
        })
        continue
      }
      log.debug(`write file ${path}`)
      writeFile(file, path)
    }
  },

  // 寻找占位
  handleHolder (file) {
    return new Promise((resolve, reject) => {
      const holderFile = getHolderFile('.');
      if (holderFile != null) {
        fs.writeFileSync(holderFile.filePath, holderFile.content.toString().replace(context.get("holder"), file.content));
        resolve(true);
        return
      }
      resolve(false)
    })
      .then(result => {
        return result
      })
  },

  // 生成代码
  generate (answers, resources, template, force) {
    log.info('正在生成代码...');
    log.debug(`projectId: ${context.getProject()}`)
    log.debug(`versionId: ${template.versionId}`)
    log.debug(`draftId: ${template.draftId}`)
    log.debug(`resources: ${resources}`)
    request.post('/cli/gen', {
      projectId: context.getProject(),
      versionId: template.versionId,
      draftId: template.draftId,
      props: answers,
      resources
    })
      .then(async previews => {
        log.debug(`generate successfully.`)
        log.debug(`file total: ${previews.length}`)
        totalFileCount = previews.length
        // 仅生成一个文件时生成至指定文件位置
        if (previews.length === 1) {
          log.debug('write code to #coderd#.')
          let handled = await this.handleHolder(previews[0]);
          if (handled) {
            log.success(`代码已生成至对应位置`);
          } else {
            log.debug('write code to #coderd# failed.')
          }
        }
        // 如果没有处理生成结果，则按文件方式进行处理
        for (const file of previews) {
          this.handleFile(file, force)
        }
        // 处理已存在的文件
        if (existsFiles.length > 0) {
          handleExistsFile()
          return
        }
        log.success(`共生成 ${totalFileCount} 个文件`)
      })
      .catch(e => {
        log.debug('generate failed, e.code = ' + e.code)
        // 不属于项目成员
        if (e.code === '000104') {
          log.error('生成失败！您不属于该项目成员。')
          process.exit()
          return
        }
        // 模版错误
        if (e.code === '020025') {
          let errorMessage = '';
          for (const tplName in e.edata) {
            errorMessage += `"${tplName}" 发生了错误：`;
            errorMessage += '\n';
            errorMessage += e.edata[tplName]
          }
          log.error(errorMessage);
          process.exit()
          return
        }
        // 模版检查错误
        if (e.code === '990000') {
          log.warn(e.message);
          process.exit()
          return
        }
        // 无法使用vip选项
        if (e.message === '尊敬的用户，你当前无法使用VIP专属配置选项') {
          log.warn(e.message);
          process.exit()
          return
        }
        log.error(e)
      })
    log.debug("send request completed.")
  }
};
